!
! Demonstration of force DLL
!
SUBROUTINE ForceDLL(time,x,xdot,xdot2,amat,omega,omegadot,F,M)
!DEC$ ATTRIBUTES DLLEXPORT::ForceDLL
!DEC$ ATTRIBUTES ALIAS:'forcedll' :: ForceDLL
! input
DOUBLE PRECISION ,INTENT(IN)                :: time            ! time
DOUBLE PRECISION ,DIMENSION(3)              :: x,x_a               ! global position of reference node
DOUBLE PRECISION ,DIMENSION(3)              :: xdot,xdot_1            ! global velocity of reference node
DOUBLE PRECISION ,DIMENSION(3) ,INTENT(IN)   :: xdot2           ! global acceleration of reference node           !     
DOUBLE PRECISION ,DIMENSION(3),INTENT(IN)    :: omega           ! angular velocity of reference node (global base)
DOUBLE PRECISION ,DIMENSION(3) ,INTENT(IN)   :: omegadot        ! angular acceleration of reference node (global base)
DOUBLE PRECISION ,DIMENSION(3,3),INTENT(IN)  :: amat            ! rotation matrix (body -> global)
! output
DOUBLE PRECISION ,DIMENSION(3) ,INTENT(OUT)   :: F               ! External force in reference node (global base)
DOUBLE PRECISION ,DIMENSION(3) ,INTENT(OUT)   :: M               ! External moment in reference node (global base)
DOUBLE PRECISION ,DIMENSION(3)                :: F1,M1               ! External moment in reference node (global base)
!
! cnvrg intitial value = 0

!-----------------------------------------------------------------------------------------------------------------
! Functions:
real(kind=8), external :: kai
! Parameters:
real(kind = 8), parameter :: pi = 3.14159265358979, thousand = 1000.0d0
real(kind = 8), parameter :: g = 9.81d0              ! acceleration of gravity
real(kind = 8), parameter :: zero = 1.0d-16, zero8 = 1.0d-8, zero4 = 1.0d-4
real(kind = 8), parameter :: dt = 0.001d0, epsilon = 1.0d-6        ! dt,time step,second; epsilon, convergence criterion
integer, parameter :: fileid = 10                                 ! file index
!integer(kind = 4), parameter :: stps_dim = 1850000                 ! maximum number of time steps
integer(kind = 4), parameter :: wl_dim = 100, wl_dim_half = 50

real(kind = 8), parameter :: R_cone = 4.0d0                        ! waterline radius
!integer(kind = 4), parameter :: print_itvl = 1000

real(kind = 8), parameter :: rhoi = 0.88d3, rhow = 1.013d3
real(kind = 8), parameter :: sigmacr = 2.3d6                         ! crushing strength
real(kind = 8), parameter :: sigmaf = 0.58d6                         ! flexual strength, model ice
real(kind = 8), parameter :: ym = 5.4d9                              ! Yong's modulus
real(kind = 8), parameter :: nu = 0.33d0                             ! Poisson ratio
real(kind = 8), parameter :: mu = 0.05d0                             ! friction coefficient
real(kind = 8), parameter :: Cl = 0.1d0, Cv = -0.10d0                ! breaking radius parameters
real(kind = 8), parameter :: kc = 3.0d0
real(kind = 8), parameter :: dl = 50.0d0
real(kind = 8), parameter :: p0 = 2.3d0                                     ! in Mpa
real(kind = 8), parameter :: A0 = 0.1225d0, A1 = 20.0d0, A2 = 200.0d0       ! in m**2
real(kind = 8), parameter :: p1_1 = 1.381771066d6
real(kind = 8), parameter :: p1_2 = 0.830127274d6
real(kind = 8), parameter :: p1_3 = 0.498726610d6
real(kind = 8), parameter :: p1_4 = 0.299627025d6
real(kind = 8), parameter :: p1_5 = 0.180003472d6
real(kind = 8), parameter :: n_pa = -0.0d0

real(kind = 8) :: hi   != 0.4d0                       ! ice thickness
real(kind = 8), parameter :: vx0 = -0.1d0                     ! ice drifting speed

integer i,j,k,ii,jj,ct
character(len=7)::fileidd

! Inputs:
!DOUBLE PRECISION,INTENT(IN)                        :: time            ! time
!DOUBLE PRECISION ,DIMENSION(3),INTENT(IN)               :: x
!DOUBLE PRECISION ,DIMENSION(2)                          :: x_a, xdot_1           ! global position of reference node
!DOUBLE PRECISION ,DIMENSION(3),INTENT(IN)                :: xdot           ! global velocity of reference node
!DOUBLE PRECISION ,DIMENSION(3),INTENT(IN)                :: xdot2           ! global acceleration of reference node           !     
!DOUBLE PRECISION ,DIMENSION(3),INTENT(IN)                :: omega           ! angular velocity of reference node (global base)
!DOUBLE PRECISION ,DIMENSION(3),INTENT(IN)                :: omegadot        ! angular acceleration of reference node (global base)
!DOUBLE PRECISION ,DIMENSION(3,3),INTENT(IN)             :: amat            ! rotation matrix (body -> global)
!real(kind = 8) :: r_cntr(2), v_cntr(2)         ! location and speed at the center of the cone waterline, in m and m/s
!real(kind = 8) :: F_cntr(6)                    ! forces and moments on the center of the cone waterline, N and Nm 
integer :: cnvrg                              ! convergence indicator: 1, converge; 0, not converged
DOUBLE PRECISION, SAVE                        :: T 
! output
!DOUBLE PRECISION ,DIMENSION(3) ,INTENT(OUT)   :: F               ! External force in reference node (global base)
!DOUBLE PRECISION ,DIMENSION(3) ,INTENT(OUT)   :: M               ! External moment in reference node (global base)

real(kind = 8) :: icenode(2,20000)             ! mm
real(kind = 8) :: icenode0(2,20000), icenode00(2,20000)
real(kind = 8) :: icenodet(2,1000000), icenodeh(2,1000000)
integer :: icenode_no_00, icenode_no_0, icenode_no
integer :: icenode_no_t, icenode_no_h
integer :: kh, kt

real(kind = 8) :: lc
real(kind = 8) :: r(6)                          ! global position and orientation in inertial cs.
real(kind = 8) :: v(6)                          ! relative speed vector, m/s.

! waterline:
real(kind = 8) :: xyoutwl(2,wl_dim), theta_cone(wl_dim)
real(kind = 8) :: phishp3(wl_dim+wl_dim_half)       ! cone slope angle

! contact region:
integer :: ipair_in_id(200), ipair_out_id(200), hpair_in_id(200), hpair_out_id(200)
real(kind = 8) :: intsct_in(2,200), intsct_out(2,200)
integer :: c_rgns

integer :: iceupdt_flag

real(kind = 8) :: xyoutwlext(2,wl_dim+wl_dim_half)
integer :: hpair_out_id_lcl
real(kind = 8) :: fshpsum(6)
integer :: iic(500), hic(500)
real(kind = 8) :: dab_hic_in,dab_hic_in1,dab_hic_out,dab_hic_out1
real(kind = 8) :: ld(200)
real(kind = 8) :: intsct_in_oneD(2), intsct_out_oneD(2)
integer :: iic_idx, hic_idx, iic_idx2, hic_idx2, hic_idx3
integer :: iic_id(200), hic_id(200)
real(kind = 8) :: test_p(2), n_pt(2)
real(kind = 8) :: ld_itmdt
integer :: cusp_id(200)
real(kind = 8) :: dab_lh
real(kind = 8) :: lh(200)
integer :: id_distance2D_lh, hic_lhi, hpair_in_id_lhi, hpair_out_id_lhi
real(kind = 8) :: hull_cntct(2,500)
real(kind = 8) :: phi_cntct(500)
real(kind = 8) :: hull_cntct_out_xtr(2), hull_cntct_in_xtr(2)
real(kind = 8) :: Acrsi(200)
integer :: cusp_id_ldi
real(kind = 8) :: pta_lhi(2), ptb_lhi(2)
real(kind = 8) :: dab_lhi
integer :: id_distance2D_lhi
real(kind = 8) :: lhi
real(kind = 8) :: cusp(2), n_pt_ldi(2)
real(kind = 8) :: dab_ldi
integer :: id_distance2D_ldi
real(kind = 8) :: ldi, phii
real(kind = 8) :: Acri(500)
real(kind = 8) :: k_pa
real(kind = 8) :: pav(200)
real(kind = 8) :: fcri(500,200)
real(kind = 8) :: fshp(3,500)
real(kind = 8) :: fcr(500)
real(kind = 8) :: fcrsum(6)
real(kind = 8) :: fvt
real(kind = 8) :: vnfip, vnlip
integer :: id_distance2D_bow, id_projection, id_projection_bow, id_projection_ldi, id_projection_theta, id_projection_cenbrk
real(kind = 8) :: a_bow, b_bow, c_bow, p_bow
real(kind = 8) :: r_in(200), r_out(200)
real(kind = 8) :: pta_theta(2), ptb_theta(2)
integer :: ipair_in_id_theta, ipair_out_id_theta
real(kind = 8) :: dr_theta
real(kind = 8) :: new_ice_in(2,200), new_ice_out(2,200)
real(kind = 8) :: oa_in_theta(2), ab_in_theta(2), oa_out_theta(2), ab_out_theta(2)
real(kind = 8) :: l_theta, m_theta, n_theta
real(kind = 8) :: t1_theta, t2_theta, x1_theta, y1_theta, x2_theta, y2_theta
real(kind = 8) :: Mid_theta(2)
real(kind = 8) :: k_theta, kp_theta, theta_in, theta_out
real(kind = 8) :: c_theta(2), v_in_theta(2), v_out_theta(2)
real(kind = 8) :: theta_wedge(200)
real(kind = 8) :: b_pf, pf, lamda
real(kind = 8) :: x_1,y_1,x_2,y_2,x_3,y_3,x_4,y_4,xc,yc
integer :: intsct_cn
real(kind = 8) :: M_cenbrk(2,200)
real(kind = 8) :: k_cenbrk, kp_cenbrk, h_mln
real(kind = 8) :: p_cenbrk(2)
real(kind = 8) :: c_cenbrk(2), in_pt(2), out_pt(2)
real(kind = 8) :: c_cenbrk_1(2), c_cenbrk_2(2)
integer :: n_new_pt
real(kind = 8) :: new_edge(2,5000)
real(kind = 8) :: fshpsum_main(6)
real(kind = 8) :: fvt_main(200)
real(kind = 8) :: Fcrbrk(6)
real(kind = 8) :: dstern1, dstern2, dst_strn, dst_cog
integer :: output_ctrl1, output_ctrl2, id1,id3
real(kind = 8) :: intsct_temp(2,400), intsct_temp2(2,400)
integer :: ipair_id_temp(400), hpair_id_temp(400), ipair_id_temp2(400), hpair_id_temp2(400)
integer :: inode_id(20000)

!integer(kind=4)::vstops                   !stoped number of time steps

real(kind = 8) :: r_check
real(kind = 8) :: dist_io
real(kind = 8) :: dist_n
integer :: dist_ctrl, dist_flag, r_check_ctrl

real(kind = 8) :: hull_lhc(2), hull_rhc(2), n_lhc(2), n_rhc(2)
integer :: id_projection_lhc, id_projection_rhc, id_distance2D_lhc, id_distance2D_rhc
real(kind = 8) :: ld_lhc, ld_rhc, a_lhc, a_rhc, b_lhc, b_rhc, c_lhc, c_rhc, p_lhc, p_rhc
real(kind = 8) :: dtheta, Rbn(3,3)

!***************************************************************************************************************
!***************************************************************************************************************
!***************************************************************************************************************
! convergence checking
LOGICAL, SAVE                    :: bInit = .FALSE. ! Initialisation flag
!initialise on first call
IF (.NOT.bInit) THEN
  bInit = .TRUE.
  T=0.0
end if
!
if (T==time) then
   cnvrg=0
else
   cnvrg=1
end if

T=time
!write(*, '(f10.3, f10.3)')    T, time
open(34, FILE="cnvrg.txt") 
write (34,'(f8.3,f8.3,f8.3)') cnvrg, T, time
!***************************************************************************************************************
!***************************************************************************************************************
IF (T.LT.1000) then
       F(1)=0  
       F(2)=77.86*T     !9.9496*T For 0.5mps0.8:110.5;For 0.1mps0.4:16.96;For 0.1mps0.8:77.86;For 0.3mps0.8: 95.06;
       F(3)=0  
       M(1)=3114*T      !35.1104*T For 0.5mps0.8:5102;For 0.1mps0.4: -436.5;For 0.1mps0.8:3114;For 0.3mps0.8: 4121;
       M(2)=0
       M(3)=0
	   write(*,301)   T
       301 format('time=',f10.3)
else

write(*,300)   T
300 format('time=',f10.3)
!***************************************************************************************************************

If (T.LT.1100) then
hi=0.0008d0*(T-1000.0)
else
hi=0.8d0
endif

!external dgesv
!external dgemm
!
! INITIALIZATION OF VARIABLES

!ct = 1
x_a(1)=-x(2)
x_a(2)=-x(1)
xdot_1(1)=-xdot(2) 
xdot_1(2)=-xdot(1)
x(1)=x_a(1)
x(2)=x_a(2)
xdot(1)=xdot_1(1) 
xdot(2)=xdot_1(2)
!r_cntr(2) = -r_cntr(2)
!v_cntr(2) = -v_cntr(2)
!r = (/0.0d0,0.0d0,0.0d0,0.0d0,0.0d0,0.0d0/)             ! structural position
!v = (/-vx0,0.0d0,0.0d0,0.0d0,0.0d0,0.0d0/)         ! relative speed
r = (/x(1),x(2),0.0d0,0.0d0,0.0d0,0.0d0/)             ! structural position
v = (/xdot(1)-vx0,xdot(2),0.0d0,0.0d0,0.0d0,0.0d0/)         ! relative speed
icenodeh = 0.0d0
icenodet = 0.0d0
icenode_no_h = 0
icenode_no_t = 0

!vstops = stps

k_pa = p0/(A0**n_pa)
lc = (ym*hi*hi*hi/(12.0d0*(1.0d0-nu*nu)*rhow*g))**0.25d0            ! in m.

!----------------------------------------------------------------------------------------------------------- INPUT FILES

! read initial ice edge
if (cnvrg==1) then        ! converged
    open(fileid,file = 'ice_nd_updated.txt')
    read(fileid,*) icenode_no_0
    close(fileid)
    open(fileid,file = 'ice_updated.txt')
    do i = 1, icenode_no_0
	    read(fileid,*) icenode0(1,i), icenode0(2,i)
    end do
    close(fileid)
else      
    open(fileid,file = 'ice_nd_initial.txt')
    read(fileid,*) icenode_no_0
    close(fileid)
    open(fileid,file = 'ice_initial.txt')
    do i = 1, icenode_no_0
	    read(fileid,*) icenode0(1,i), icenode0(2,i)
    end do
    close(fileid)
end if

icenode00 = icenode0
icenode_no_00 = icenode_no_0

forall (j = 1:icenode_no_0)
    icenode0(1,j) = icenode0(1,j) + dt*vx0*thousand
end forall

! structure
dtheta = pi/50.0d0
do j = 1,wl_dim
    theta_cone(j) = pi+dble(j-1)*dtheta
    !xyoutwl(1,j) = R_cone*dcos(theta_cone(j))*thousand
    !xyoutwl(2,j) = R_cone*dsin(theta_cone(j))*thousand
	xyoutwl(1,j) = R_cone*dcos(theta_cone(j))*thousand + x(1)*thousand
    xyoutwl(2,j) = R_cone*dsin(theta_cone(j))*thousand + x(2)*thousand
end do
phishp3 = 1.0d0*pi/4.0d0  ! cone angle =45degree
!phishp3 = 11.0d0*pi/36.0d0  ! cone angle =55degree
!phishp3 = 13.0d0*pi/36.0d0  ! cone angle =65degree

!open(fileid,file="g0.dat")
!do j=1,wl_dim
!	write(fileid,'(2f20.5)')xyoutwl(1,j)/1000.0d0,xyoutwl(2,j)/1000.0d0
!end do
!write(fileid,'(2f20.5)')xyoutwl(1,1)/1000.0d0,xyoutwl(2,1)/1000.0d0
!write(fileid,*)
!do j=1,icenode_no_0
!	write(fileid,'(2f20.5)')icenode0(1,j)/1000.0d0,icenode0(2,j)/1000.0d0
!end do
!close(fileid)
!--------------------------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------------------------
! INITIAL VALUES:

Rbn = 0.0d0
do j = 1,3 
    Rbn(j,j) = 1.0d0
end do
                        
forall (i = 1:2,j = 1:wl_dim)
    xyoutwlext(i,j) = xyoutwl(i,j)
end forall
forall (i = 1:2,j = wl_dim+1:wl_dim+wl_dim_half)
    xyoutwlext(i,j) = xyoutwl(i,j-wl_dim)
end forall
              
call contact_zones(xyoutwl,icenode0,icenode_no_0,ipair_in_id,ipair_out_id,hpair_in_id,hpair_out_id,intsct_in,intsct_out,lh,c_rgns,id1,id3,ipair_id_temp,hpair_id_temp,ipair_id_temp2,hpair_id_temp2,intsct_temp,intsct_temp2,inode_id)
	    
!if (stpi==10000000000) then
!    open(454545,file='ice-initial.txt')
!    do i = 1,icenode_no_0
!        write(454545,*) icenode0(1,i),icenode0(2,i)
!    end do
!    close(454545)
    
!    open(898989,file='ship-contact.txt')
!    do i = 1,wl_dim
!        write(898989,*) xyoutwl(1,i),xyoutwl(2,i)
!    end do
!    close(898989)
    
!    open(909090,file='inode_id.txt')
!    do i = 1,icenode_no_0
!        write(909090,*) inode_id(i)
!    end do
!    close(909090)
    
!    open(787878,file='contact-info.txt')
!    write(787878,*) 'id1,id3:',id1,id3
!    write(787878,*) 'c_rgns:',c_rgns
!    write(787878,*)
!    do i = 1,id1
!        write(787878,*) ipair_id_temp(i), hpair_id_temp(i), intsct_temp(1,i),intsct_temp(2,i)
!        write(787878,*)
!    end do
!    write(787878,*)
!    do i = 1,id3
!        write(787878,*) ipair_id_temp2(i), hpair_id_temp2(i), intsct_temp2(1,i),intsct_temp2(2,i)
!        write(787878,*)
!    end do
!    write(787878,*) 'ipair_in_id,ipair_out_id,hpair_in_id,hpair_out_id:'
!    do i = 1,c_rgns
!        write(787878,*) i
!        write(787878,*) ipair_in_id(i),ipair_out_id(i), hpair_in_id(i),hpair_out_id(i)
!        write(787878,*) intsct_in(1,i),intsct_in(2,i), intsct_out(1,i),intsct_out(2,i)
!        write(787878,*) 
!    end do
!    close(787878)
    !pause
    !stop
!end if
    
output_ctrl1 = 0
if (output_ctrl1 == 1) then
    write(*,*) 'c_rgns',c_rgns
    !pause
    if (c_rgns>=1) then
        do i = 1,c_rgns
            write(*,*) 'i=',i
            write(*,*) 'ipair_in_id',ipair_in_id(i)
            write(*,*) 'ipair_out_id',ipair_out_id(i)
            write(*,*) 'hpair_in_id',hpair_in_id(i)
            write(*,*) 'hpair_out_id',hpair_out_id(i)
            write(*,*) 'intsct_in',intsct_in(1,i),intsct_in(2,i)
            write(*,*) 'intsct_out',intsct_out(1,i),intsct_out(2,i)
            write(*,*) 'lh',lh(i)
            write(*,*)
            !pause
        end do
    end if
    !goto 111
end if
!--------------------------------------------------------------------
!------------------------------------------------------------------- |
!               Ice Crushing & breaking Force     Start             ||
!------------------------------------------------------------------- |
!--------------------------------------------------------------------

fshpsum = 0.0d0

if (c_rgns == 0) then                      ! level 1
    Fcrbrk = 0.0d0                          ! in N.
else        
    ! count number of ice nodes in each contact zone(iic(500)), hull nodes in each contact zone(hic(500))
    do i = 1,c_rgns
        
        dab_hic_in = dsqrt((intsct_in(1,i)-icenode0(1,ipair_in_id(i)))*(intsct_in(1,i)-icenode0(1,ipair_in_id(i)))+(intsct_in(2,i)-icenode0(2,ipair_in_id(i)))*(intsct_in(2,i)-icenode0(2,ipair_in_id(i))))
        dab_hic_in1 = dsqrt((intsct_in(1,i)-icenode0(1,ipair_in_id(i)+1))*(intsct_in(1,i)-icenode0(1,ipair_in_id(i)+1))+(intsct_in(2,i)-icenode0(2,ipair_in_id(i)+1))*(intsct_in(2,i)-icenode0(2,ipair_in_id(i)+1)))
        dab_hic_out = dsqrt((intsct_out(1,i)-icenode0(1,ipair_out_id(i)))*(intsct_out(1,i)-icenode0(1,ipair_out_id(i)))+(intsct_out(2,i)-icenode0(2,ipair_out_id(i)))*(intsct_out(2,i)-icenode0(2,ipair_out_id(i))))
        dab_hic_out1 = dsqrt((intsct_out(1,i)-icenode0(1,ipair_out_id(i)+1))*(intsct_out(1,i)-icenode0(1,ipair_out_id(i)+1))+(intsct_out(2,i)-icenode0(2,ipair_out_id(i)+1))*(intsct_out(2,i)-icenode0(2,ipair_out_id(i)+1)))
        if ((dab_hic_in<=zero8).and.(dab_hic_out<=zero8)) then
            iic(i) = ipair_out_id(i) - ipair_in_id(i) - 1                       ! iic [0,...]
            iic_id(i) = 11
        else if ((dab_hic_in<=zero8).and.(dab_hic_out1<=zero8)) then
            iic(i) = ipair_out_id(i) - ipair_in_id(i)
            iic_id(i) = 12
        else if ((dab_hic_in<=zero8).and.(dab_hic_out>zero8).and.(dab_hic_out1>zero8)) then
            iic(i) = ipair_out_id(i) - ipair_in_id(i)
            iic_id(i) = 13
        else if ((dab_hic_in1<=zero8).and.(dab_hic_out<=zero8)) then
            iic(i) = ipair_out_id(i) - ipair_in_id(i) - 2
            iic_id(i) = 21
        else if ((dab_hic_in1<=zero8).and.(dab_hic_out1<=zero8)) then
            iic(i) = ipair_out_id(i) - ipair_in_id(i) - 1
            iic_id(i) = 22
        else if ((dab_hic_in1<=zero8).and.(dab_hic_out>zero8).and.(dab_hic_out1>zero8)) then
            iic(i) = ipair_out_id(i) - ipair_in_id(i) - 1
            iic_id(i) = 23
        else if ((dab_hic_out<=zero8).and.(dab_hic_in1>zero8).and.(dab_hic_in>zero8)) then
            iic(i) = ipair_out_id(i) - ipair_in_id(i) - 1
            iic_id(i) = 31
        else if ((dab_hic_in1>zero8).and.(dab_hic_in>zero8).and.(dab_hic_out1<=zero8)) then
            iic(i) = ipair_out_id(i) - ipair_in_id(i)
            iic_id(i) = 32
        else if ((dab_hic_in1>zero8).and.(dab_hic_in>zero8).and.(dab_hic_out>zero8).and.(dab_hic_out1>zero8)) then
            iic(i) = ipair_out_id(i) - ipair_in_id(i)
            iic_id(i) = 33
        else
            write(*,*) 'em01'
            !pause
        end if
        
        ! hic [0,...]
        dab_hic_in = dsqrt((intsct_in(1,i)-xyoutwlext(1,hpair_in_id(i)))*(intsct_in(1,i)-xyoutwlext(1,hpair_in_id(i)))+(intsct_in(2,i)-xyoutwlext(2,hpair_in_id(i)))*(intsct_in(2,i)-xyoutwlext(2,hpair_in_id(i))))
        dab_hic_in1 = dsqrt((intsct_in(1,i)-xyoutwlext(1,hpair_in_id(i)+1))*(intsct_in(1,i)-xyoutwlext(1,hpair_in_id(i)+1))+(intsct_in(2,i)-xyoutwlext(2,hpair_in_id(i)+1))*(intsct_in(2,i)-xyoutwlext(2,hpair_in_id(i)+1)))
        dab_hic_out = dsqrt((intsct_out(1,i)-xyoutwlext(1,hpair_out_id(i)))*(intsct_out(1,i)-xyoutwlext(1,hpair_out_id(i)))+(intsct_out(2,i)-xyoutwlext(2,hpair_out_id(i)))*(intsct_out(2,i)-xyoutwlext(2,hpair_out_id(i))))
        dab_hic_out1 = dsqrt((intsct_out(1,i)-xyoutwlext(1,hpair_out_id(i)+1))*(intsct_out(1,i)-xyoutwlext(1,hpair_out_id(i)+1))+(intsct_out(2,i)-xyoutwlext(2,hpair_out_id(i)+1))*(intsct_out(2,i)-xyoutwlext(2,hpair_out_id(i)+1)))
        
        if (hpair_out_id(i)>=hpair_in_id(i)) then
            hpair_out_id_lcl = hpair_out_id(i)
        else
            hpair_out_id_lcl = hpair_out_id(i)+wl_dim
        end if
        
        if ((dab_hic_in<=zero8).and.(dab_hic_out<=zero8)) then
            hic(i) = hpair_out_id_lcl - hpair_in_id(i) - 1
            hic_id(i) = 11
        else if ((dab_hic_in<=zero8).and.(dab_hic_out1<=zero8)) then
            hic(i) = hpair_out_id_lcl - hpair_in_id(i)
            hic_id(i) = 12
        else if ((dab_hic_in<=zero8).and.(dab_hic_out>zero8).and.(dab_hic_out1>zero8)) then
            hic(i) = hpair_out_id_lcl - hpair_in_id(i)
            hic_id(i) = 13
        else if ((dab_hic_in1<=zero8).and.(dab_hic_out<=zero8)) then
            hic(i) = hpair_out_id_lcl - hpair_in_id(i) - 2
            hic_id(i) = 21
        else if ((dab_hic_in1<=zero8).and.(dab_hic_out1<=zero8)) then
            hic(i) = hpair_out_id_lcl - hpair_in_id(i) - 1
            hic_id(i) = 22
        else if ((dab_hic_in1<=zero8).and.(dab_hic_out>zero8).and.(dab_hic_out1>zero8)) then
            hic(i) = hpair_out_id_lcl - hpair_in_id(i) - 1
            hic_id(i) = 23
        else if ((dab_hic_in>zero8).and.(dab_hic_in1>zero8).and.(dab_hic_out<=zero8)) then
            hic(i) = hpair_out_id_lcl - hpair_in_id(i) - 1
            hic_id(i) = 31
        else if ((dab_hic_in>zero8).and.(dab_hic_in1>zero8).and.(dab_hic_out1<=zero8)) then
            hic(i) = hpair_out_id_lcl - hpair_in_id(i)
            hic_id(i) = 32
        else if ((dab_hic_in>zero8).and.(dab_hic_in1>zero8).and.(dab_hic_out>zero8).and.(dab_hic_out1>zero8)) then
            hic(i) = hpair_out_id_lcl - hpair_in_id(i)
            hic_id(i) = 33
        else
            write(*,*) 'em02'
            !pause
        end if
    end do
    
    !if (stpi==90000000000) then
    !    open(7878798,file='hic-info.txt')
    !    do i = 1,c_rgns
    !        write(7878798,*) i
    !        write(7878798,*) hic(i),iic(i),lh(i)
    !    end do
    !    close(7878798)
    !end if

    ld = -1.0d0
    cusp_id = -1
    fcri = 0.0d0

    do i = 1,c_rgns
        if (lh(i)<=zero8) then
            continue
        else
            hull_cntct = 1.23456789d0
            phi_cntct = 4.0d0

            ! get crushing deepth ld, identify cusp point                
            intsct_in_oneD(1) = intsct_in(1,i)              ! mm
            intsct_in_oneD(2) = intsct_in(2,i)
            intsct_out_oneD(1) = intsct_out(1,i)
            intsct_out_oneD(2) = intsct_out(2,i)
        
            if (iic(i)/=0) then
                if ((iic_id(i) == 11).or.(iic_id(i) == 12).or.(iic_id(i) == 13).or.(iic_id(i) == 31).or.(iic_id(i) == 32).or.(iic_id(i) == 33)) then
                    iic_idx2 = ipair_in_id(i)
                else if ((iic_id(i) == 21).or.(iic_id(i) == 22).or.(iic_id(i) == 23)) then
                    iic_idx2 = ipair_in_id(i) + 1
                end if
                
                do j = 1,iic(i)
                    iic_idx = iic_idx2 + j                  ! ### needs improvements
                    test_p(1) = icenode0(1,iic_idx)
                    test_p(2) = icenode0(2,iic_idx)
                    call projection2D(intsct_in_oneD,intsct_out_oneD,test_p,n_pt,id_projection)
                    ! n_pt,the perpendicular point to project ice nodes inside each contact zone
                    if (id_projection == 1) then
                        ld_itmdt = dsqrt((test_p(1)-n_pt(1))*(test_p(1)-n_pt(1))+(test_p(2)-n_pt(2))*(test_p(2)-n_pt(2)))
                        if (ld_itmdt>ld(i)) then
                            ld(i) = ld_itmdt                        ! ld in mm.
                            cusp_id(i) = iic_idx
                        end if
                    else if (id_projection == -1) then
                        if (cusp_id(i)>0) then
                            continue
                        else
                            cusp_id(i) = -2
                            ld(i) = -2.0d0
                        end if
                    else
                        write(*,*) 'em03'
                        open(20201,file='ice_m03.txt')
                            do k = 1,icenode_no_0
                                write(20201,*) icenode0(1,k),icenode0(2,k)
                            end do
                        close(20201)
                        open(20200,file='ship_m03.txt')
                            do k = 1,wl_dim
                                write(20200,*) xyoutwl(1,k), xyoutwl(2,k)
                            end do
                        close(20200)
                        open(20202,file='m03.txt')
                        !write(20202,*) 'stpi',stpi
                        write(20202,*) 'c_rgns:',c_rgns
                        write(20202,*) 'i:',i
                        write(20202,*) 'iic(i)',iic(i)
                        write(20202,*) 'j:',j
                        write(20202,*) 'iic_idx2:',iic_idx2
                        write(20202,*) 'intsct_in_oneD:',intsct_in_oneD
                        write(20202,*) 'intsct_out_oneD:',intsct_out_oneD
                        write(20202,*) 'id_projection:',id_projection
                        close(20202)
                        !pause
                    end if
                end do
            else
                cusp_id(i) = 0      ! ### this will affect Acri. cusp_id(i) == 0 means iic == 0, could be used as identification of contact status
                ld(i) = -5.0d0
                !write(*,*) 'warning! 0002'
                !pause
                !goto 111
            end if
            
            !if (stpi == 9000000000) then
            !    write(*,*) iic_idx2,cusp_id(i),ld(i)  
                !pause
            !end if
            
            if (cusp_id(i) == -1) then
                write(*,*) 'em04'
                !write(*,*) 'stpi',stpi
                !pause
            end if
                        
        ! rearrange the hull nodes in contact zones including intersction points
            hic_lhi = hic(i)
            hpair_in_id_lhi = hpair_in_id(i)
            hpair_out_id_lhi = hpair_out_id(i)
            
            hull_cntct(1,1) = intsct_in(1,i)
            hull_cntct(2,1) = intsct_in(2,i)
            phi_cntct(1) = phishp3(1)

            if (hic_lhi/=0) then
                if ((hic_id(i) == 11).or.(hic_id(i) == 12).or.(hic_id(i) == 13).or.(hic_id(i) == 31).or.(hic_id(i) == 32).or.(hic_id(i) == 33)) then
                    hic_idx2 = hpair_in_id_lhi + 1
                else if ((hic_id(i) == 21).or.(hic_id(i) == 22).or.(hic_id(i) == 23)) then
                    hic_idx2 = hpair_in_id_lhi + 2
                end if
                
                do j = 2,hic_lhi+1                          ! 2, hic_lhi+2-1
                    hic_idx = hic_idx2 + j - 2
                    hull_cntct(1,j) = xyoutwlext(1,hic_idx)
                    hull_cntct(2,j) = xyoutwlext(2,hic_idx)
                    phi_cntct(j) = phishp3(1)
                end do
            end if
            
            hull_cntct(1,hic_lhi+2) = intsct_out(1,i)
            hull_cntct(2,hic_lhi+2) = intsct_out(2,i)                
            phi_cntct(hic_lhi+2) = phishp3(1)
            
            hull_cntct_out_xtr(1) = xyoutwlext(1,hpair_out_id_lhi+1)
            hull_cntct_out_xtr(2) = xyoutwlext(2,hpair_out_id_lhi+1)
                            
            hull_cntct_in_xtr(1) = xyoutwl(1,hpair_in_id_lhi)
            hull_cntct_in_xtr(2) = xyoutwl(2,hpair_in_id_lhi)
            
            !if (stpi==9000000000) then
            !    open(787,file='hull-rearrange-info.txt')
            !    do j = 1,c_rgns
            !        do k = 1,hic_lhi+2
            !            write(787,*) hull_cntct(1,k),hull_cntct(2,k)
            !            write(787,*) phi_cntct(k)
            !            write(787,*)
            !        end do
            !        write(787,*)
            !        write(787,*) hull_cntct_out_xtr
            !        write(787,*) hull_cntct_in_xtr
            !    end do
            !    close(787)
            !end if             
            
            do j = 1,hic_lhi+2
                if (phi_cntct(j)==4.0d0) then
                    write(*,*) 'em05'
                    write(*,*) 'j=',j
                    !pause
                    !vstops = stpi-1
                    !goto 111
                end if
                
                if ((hull_cntct(1,j) == 1.23456789d0).and.(hull_cntct(2,j) == 1.23456789d0)) then
                    write(*,*) 'em06'
                    write(*,*) 'j=',j
                    !vstops = stpi-1
                    !pause
                    !goto 111
                end if
            end do

        ! CONTACT AREA  
        ! sub crushing length, lhi; crushing depth, ldi, on each sub contact zone within each contact zone
            if (cusp_id(i) == 0.or.cusp_id(i) == -2) then       ! cusp_id == -2 needs refinement
                Acrsi(i) = 0.0d0
            else
                Acrsi(i) = 0.0d0
                cusp_id_ldi = cusp_id(i)
                do j = 1,hic_lhi+1                      ! pieces of cheese in each contact zone
                    pta_lhi(1) = hull_cntct(1,j)
                    pta_lhi(2) = hull_cntct(2,j)
                    ptb_lhi(1) = hull_cntct(1,j+1)
                    ptb_lhi(2) = hull_cntct(2,j+1)
                    call distance2D(pta_lhi,ptb_lhi,dab_lhi,id_distance2D_lhi)
                    lhi = dab_lhi                  ! sub contact length, lhi, real, mm
                    if (id_distance2D_lhi == 1) then
                        write(*,*) 'em07'
                        !vstops = stpi-1
                        !pause
                        !goto 111
                    end if

                    cusp(1) = icenode0(1,cusp_id_ldi)        ! coordiantes of ice cusp
                    cusp(2) = icenode0(2,cusp_id_ldi)
                    call projection2D(pta_lhi,ptb_lhi,cusp,n_pt_ldi,id_projection_ldi)
                    call distance2D(cusp,n_pt_ldi,dab_ldi,id_distance2D_ldi)
                    ldi = dab_ldi                        ! sub contact depth, ldi, in each cheese, mm
                    if (id_distance2D_ldi == 1) then
                        !write(*,*) 'warning m08'
                        !vstops = stpi-1
                        !pause
                        !goto 111
                        Acri(j) = 0.0d0
                        cycle
                    end if
        
                    phii = phi_cntct(1)       ! frame angle for each piece 
                    
                    if (ldi*dtan(phii)>(hi*1000.0d0)) then
                        Acri(j) = 0.5d0*(hi*1000.0d0)/dsin(phii)*(lhi+lhi*(1.0d0-hi*1000.0d0/(ldi*dtan(phii))))
                    else
                        Acri(j) = 0.5d0*lhi*ldi/dcos(phii)
                    end if
            
                    Acrsi(i) = Acrsi(i)+Acri(j)           ! total area of each contact zone, in mm**2
                    
                    !if (stpi==90000000000) then
                    !    open(787,file='Area-info.txt')
                    !    write(787,*) 'lhi:',lhi
                    !    write(787,*) 'cusp:',cusp
                    !    write(787,*) 'ldi:',ldi
                    !    write(787,*) 'id_distance2D_ldi:',id_distance2D_ldi
                    !    write(787,*) 'phii:',phii
                    !    write(787,*) 'ldi*dtan(phii)',ldi*dtan(phii)
                    !    write(787,*) 'Acri:',Acri(j)
                    !    write(787,*) 'Acrsi:',Acrsi(i)
                    !    close(787)
                    !    pause
                    !end if                              
        
                end do
            end if
            
        ! average crushing pressure
            if ((Acrsi(i)>=A0*1.0d6).and.(Acrsi(i)<=A1*1.0d6)) then       
                !write(*,*) 'Acrsi(i)>A0'
                !pause
                pav(i) = (k_pa*(Acrsi(i)/1.0d6)**n_pa)*1.0d6
            else if ((Acrsi(i)<A0*1.0d6).and.(Acrsi(i)>zero)) then
                pav(i) = sigmacr                   !the value of P0
            else if ((Acrsi(i)>A1*1.0d6).and.(Acrsi(i)<=A2*1.0d6)) then
                if (n_pa == -0.1d0) then
                    pav(i) = p1_1
                else if (n_pa == -0.2d0) then
                    pav(i) = p1_2
                else if (n_pa == -0.3d0) then
                    pav(i) = p1_3
                else if (n_pa == -0.4d0) then
                    pav(i) = p1_4
                else if (n_pa == -0.5d0) then
                    pav(i) = p1_5
                end if
            else if ((Acrsi(i)<=zero).and.(Acrsi(i)>=0.0d0)) then
                pav(i) = 0.0d0
            else
                if (n_pa == -0.1d0) then
                    pav(i) = p1_1
                else if (n_pa == -0.2d0) then
                    pav(i) = p1_2
                else if (n_pa == -0.3d0) then
                    pav(i) = p1_3
                else if (n_pa == -0.4d0) then
                    pav(i) = p1_4
                else if (n_pa == -0.5d0) then
                    pav(i) = p1_5
                end if                        
                !pav(i) = sigmacr
                write(*,*) 'em10'
                write(*,*) r(6)*180.0d0/pi
                !pause
                
                open(454545,file='ice-m10.txt')
                do j = 1,icenode_no_0
                    write(454545,*) icenode0(1,j),icenode0(2,j)
                end do
                close(454545)
    	        
                open(898989,file='ship-m10.txt')
                do j = 1,wl_dim
                    write(898989,*) xyoutwl(1,j),xyoutwl(2,j)
                end do
                close(898989)
    	        
                open(909090,file='inode_id-m10.txt')
                do j = 1,icenode_no_0
                    write(909090,*) inode_id(j)
                end do
                close(909090)
    	        
                open(787878,file='Acrsi-m10.txt')
                !write(787878,*) 'stpi,i:',stpi,i
                write(787878,*) 'id1,id3:',id1,id3
                write(787878,*) 'c_rgns:',c_rgns
                write(787878,*) 'Acrsi',Acrsi(i)/1.0d6
                write(787878,*)
                do j = 1,id1
                    write(787878,*) ipair_id_temp(j), hpair_id_temp(j), intsct_temp(1,j),intsct_temp(2,j)
                    write(787878,*)
                end do
                write(787878,*)
                do j = 1,id3
                    write(787878,*) ipair_id_temp2(j), hpair_id_temp2(j), intsct_temp2(1,j),intsct_temp2(2,j)
                    write(787878,*)
                end do
                write(787878,*) 'ipair_in_id,ipair_out_id,hpair_in_id,hpair_out_id:'
                do j = 1,c_rgns
                    write(787878,*) j
                    write(787878,*) ipair_in_id(j),ipair_out_id(j), hpair_in_id(j),hpair_out_id(j)
                    write(787878,*) intsct_in(1,j),intsct_in(2,j), intsct_out(1,j),intsct_out(2,j)
                    write(787878,*) 
                end do
                close(787878)
                write(*,*) 'output finished'
                !stop
            end if                    
             
        ! crushing force on patches
            do j = 1,hic_lhi+1
                fcri(j,i) = pav(i)*Acri(j)/1.0d6      ! curshing force fcri(j,i) on each piece, in N
            end do
            
        ! nodal force, to be used in the force decomposition subroutine
            fcr(1) = 0.5d0*fcri(1,i)
            do j = 2,hic_lhi+1
                fcr(j) = 0.5d0*(fcri(j-1,i)+fcri(j,i))
            end do
            fcr(hic_lhi+2) = 0.5d0*(fcri(hic_lhi+1,i))
            !write(*,*) 'mark'
            !pause
            
            output_ctrl2 = 0
            if (output_ctrl2 == 1) then
                do j = 1,hic_lhi+1
                    if (fcr(j)==0.0d0) then
                        write(*,*) 'em11'
                        write(*,*) 'fcr(',j,')=',fcr(j)
                        write(*,*) 'fcri(',j,i,')=',fcri(j,i)
                        write(*,*) 'Acrsi(',i,')=', Acrsi(i)
                        write(*,*) 'Acri(',j,i,')=',Acri(j)
                        write(*,*) 'pav(',i,')=',pav(i)
                        write(*,*) 'hic_lhi+1',hic_lhi+1
                        !vstops = stpi-1
                        !pause
                        !goto 111
                    end if
                end do
            end if  
            
            !if (stpi==90000000000) then
            !    open(787,file='fcr-info.txt')
            !    write(787,*) 'pav:',pav(i)
            !    write(787,*)
            !    do j = 1,hic_lhi+1
            !        write(787,*) 'fcri:',fcri(j,i)
            !    end do
            !    do j = 1,hic_lhi+2
            !        write(787,*) 'fcr:',fcr(j)
            !    end do
            !    close(787)
                !pause
            !end if                
            
            fshpsum_main = 0.0d0
            call forcecomp(mu,r,v,Rbn,hic_lhi,hull_cntct,phi_cntct,hull_cntct_in_xtr,hull_cntct_out_xtr,fcr,fshpsum_main,fvt,vnfip,vnlip,fshp)
            
            fvt_main(i) = dabs(fvt)
            !if (stpi==9000000000) then
            !    open(787,file='fvt-info.txt')
            !    write(787,*) 'fvt, fvt_main:',fvt, fvt_main(i)
            !    close(787)
            !end if                 
                            
            ! breaking radius, new ice edge points
            r_in(i) = Cl*lc*(1.0d0+Cv*dabs(vnfip))*thousand                 ! breaking radius
        
            pta_theta(1) = intsct_in(1,i)
            pta_theta(2) = intsct_in(2,i)
            ipair_in_id_theta = ipair_in_id(i)
            ptb_theta(1) = icenode0(1,ipair_in_id_theta)
            ptb_theta(2) = icenode0(2,ipair_in_id_theta)
            dr_theta = dsqrt((ptb_theta(1)-pta_theta(1))*(ptb_theta(1)-pta_theta(1))+(ptb_theta(2)-pta_theta(2))*(ptb_theta(2)-pta_theta(2)))
            if (dr_theta>=r_in(i)) then
                new_ice_in(1,i) = pta_theta(1)+(ptb_theta(1)-pta_theta(1))*r_in(i)/dr_theta
                new_ice_in(2,i) = pta_theta(2)+(ptb_theta(2)-pta_theta(2))*r_in(i)/dr_theta
            else
                do while (dr_theta<r_in(i))
                    ipair_in_id_theta = ipair_in_id_theta - 1
                    ptb_theta(1) = icenode0(1,ipair_in_id_theta)
                    ptb_theta(2) = icenode0(2,ipair_in_id_theta)
                    dr_theta = dsqrt((ptb_theta(1)-pta_theta(1))*(ptb_theta(1)-pta_theta(1))+(ptb_theta(2)-pta_theta(2))*(ptb_theta(2)-pta_theta(2)))
                end do
        
                oa_in_theta(1) = icenode0(1,ipair_in_id_theta) - pta_theta(1)            ! vector oa
                oa_in_theta(2) = icenode0(2,ipair_in_id_theta) - pta_theta(2)
                ab_in_theta(1) = icenode0(1,ipair_in_id_theta+1) - icenode0(1,ipair_in_id_theta)      ! vector ab
                ab_in_theta(2) = icenode0(2,ipair_in_id_theta+1) - icenode0(2,ipair_in_id_theta)
                l_theta = oa_in_theta(1)*oa_in_theta(1) + oa_in_theta(2)*oa_in_theta(2) - r_in(i)*r_in(i)
                m_theta = oa_in_theta(1)*ab_in_theta(1) + oa_in_theta(2)*ab_in_theta(2)
                n_theta = ab_in_theta(1)*ab_in_theta(1) + ab_in_theta(2)*ab_in_theta(2)
                if (n_theta*n_theta<=zero) then
                    write(*,*) 'em12'
                    !vstops = stpi-1
                    !pause
                    !goto 111
                end if
                if ((m_theta*m_theta-n_theta*l_theta)<-zero) then
                    write(*,*) 'em13'
                    write(*,*) 'm**2-n*l*=',m_theta*m_theta-n_theta*l_theta
                    write(*,*) 'pta_theta, ptb_theta,r_in(i),dr_theta,icenode0(:,ipair_in_id_theta+1)'
                    write(*,*) pta_theta,ptb_theta,r_in(i),dr_theta,icenode0(1,ipair_in_id_theta+1),icenode0(2,ipair_in_id_theta+1)
                    !vstops = stpi-1
                    !pause
                    !goto 111
                else if (dabs(m_theta*m_theta-n_theta*l_theta)<=zero) then
                    t1_theta = -m_theta/n_theta
                    t2_theta = t1_theta
                else
                    t1_theta = (-m_theta+dsqrt(m_theta*m_theta-n_theta*l_theta))/n_theta
                    t2_theta = (-m_theta-dsqrt(m_theta*m_theta-n_theta*l_theta))/n_theta
                end if
                x1_theta = icenode0(1,ipair_in_id_theta) + ab_in_theta(1)*t1_theta
                y1_theta = icenode0(2,ipair_in_id_theta) + ab_in_theta(2)*t1_theta
                x2_theta = icenode0(1,ipair_in_id_theta) + ab_in_theta(1)*t2_theta
                y2_theta = icenode0(2,ipair_in_id_theta) + ab_in_theta(2)*t2_theta

                if ((((x1_theta-icenode0(1,ipair_in_id_theta))*(x1_theta-icenode0(1,ipair_in_id_theta+1)))<=0.0d0).and.(((y1_theta-icenode0(2,ipair_in_id_theta))*(y1_theta-icenode0(2,ipair_in_id_theta+1)))<=0.0d0)) then
                    if ((((x2_theta-icenode0(1,ipair_in_id_theta))*(x2_theta-icenode0(1,ipair_in_id_theta+1)))<=0.0d0).and.(((y2_theta-icenode0(2,ipair_in_id_theta))*(y2_theta-icenode0(2,ipair_in_id_theta+1)))<=0.0d0).and.(dabs(t1_theta-t2_theta)>zero)) then
                        write(*,*) 'em14'        ! two intersctions between section and circle
                        !vstops = stpi-1
                        !pause
                        !goto 111
                    else
                        new_ice_in(1,i) = x1_theta
                        new_ice_in(2,i) = y1_theta
                    end if
                else
                    if ((((x2_theta-icenode0(1,ipair_in_id_theta))*(x2_theta-icenode0(1,ipair_in_id_theta+1)))<=0.0d0).and.(((y2_theta-icenode0(2,ipair_in_id_theta))*(y2_theta-icenode0(2,ipair_in_id_theta+1)))<=0.0d0)) then
                        new_ice_in(1,i) = x2_theta
                        new_ice_in(2,i) = y2_theta
                    else
                        write(*,*) 'em15'
                        !vstops = stpi-1
                        !pause
                        !goto 111
                    end if
                end if
            end if
            
            !write(*,*) 'mark2'
            !pause
            
            r_out(i) = Cl*lc*(1.0d0+Cv*dabs(vnlip))*thousand        ! breaking radius - out
            
            pta_theta(1) = intsct_out(1,i)
            pta_theta(2) = intsct_out(2,i)
            ipair_out_id_theta = ipair_out_id(i)+1
            ptb_theta(1) = icenode0(1,ipair_out_id_theta)
            ptb_theta(2) = icenode0(2,ipair_out_id_theta)
            dr_theta = dsqrt((ptb_theta(1)-pta_theta(1))*(ptb_theta(1)-pta_theta(1))+(ptb_theta(2)-pta_theta(2))*(ptb_theta(2)-pta_theta(2)))
            
            if (dr_theta>=r_out(i)) then
                new_ice_out(1,i) = pta_theta(1)+(ptb_theta(1)-pta_theta(1))*r_out(i)/dr_theta
                new_ice_out(2,i) = pta_theta(2)+(ptb_theta(2)-pta_theta(2))*r_out(i)/dr_theta
            else
                do while (dr_theta<r_out(i))
                    ipair_out_id_theta = ipair_out_id_theta + 1
                    ptb_theta(1) = icenode0(1,ipair_out_id_theta)
                    ptb_theta(2) = icenode0(2,ipair_out_id_theta)
                    dr_theta = dsqrt((ptb_theta(1)-pta_theta(1))*(ptb_theta(1)-pta_theta(1))+(ptb_theta(2)-pta_theta(2))*(ptb_theta(2)-pta_theta(2)))
                end do
        
                oa_out_theta(1) = icenode0(1,ipair_out_id_theta) - pta_theta(1)            ! vector oa
                oa_out_theta(2) = icenode0(2,ipair_out_id_theta) - pta_theta(2)
                ab_out_theta(1) = icenode0(1,ipair_out_id_theta-1) - icenode0(1,ipair_out_id_theta)      ! vector ab
                ab_out_theta(2) = icenode0(2,ipair_out_id_theta-1) - icenode0(2,ipair_out_id_theta)
                l_theta = oa_out_theta(1)*oa_out_theta(1) + oa_out_theta(2)*oa_out_theta(2) - r_out(i)*r_out(i)
                m_theta = oa_out_theta(1)*ab_out_theta(1) + oa_out_theta(2)*ab_out_theta(2)
                n_theta = ab_out_theta(1)*ab_out_theta(1) + ab_out_theta(2)*ab_out_theta(2)
                if (n_theta*n_theta<=zero) then
                    write(*,*) 'em16'
                    !vstops = stpi-1
                    !pause
                    !goto 111
                end if
                if ((m_theta*m_theta-n_theta*l_theta)<-zero) then
                    write(*,*) 'em17'
                    !vstops = stpi-1
                    !pause
                    !goto 111
                else if (dabs(m_theta*m_theta-n_theta*l_theta)<=zero) then
                    t1_theta = -m_theta/n_theta
                    t2_theta = t1_theta
                else
                    t1_theta = (-m_theta+dsqrt(m_theta*m_theta-n_theta*l_theta))/n_theta
                    t2_theta = (-m_theta-dsqrt(m_theta*m_theta-n_theta*l_theta))/n_theta
                end if
                x1_theta = icenode0(1,ipair_out_id_theta) + ab_out_theta(1)*t1_theta
                y1_theta = icenode0(2,ipair_out_id_theta) + ab_out_theta(2)*t1_theta
                x2_theta = icenode0(1,ipair_out_id_theta) + ab_out_theta(1)*t2_theta
                y2_theta = icenode0(2,ipair_out_id_theta) + ab_out_theta(2)*t2_theta

                if ((((x1_theta-icenode0(1,ipair_out_id_theta))*(x1_theta-icenode0(1,ipair_out_id_theta-1)))<=0.0d0).and.(((y1_theta-icenode0(2,ipair_out_id_theta))*(y1_theta-icenode0(2,ipair_out_id_theta-1)))<=0.0d0)) then
                    if ((((x2_theta-icenode0(1,ipair_out_id_theta))*(x2_theta-icenode0(1,ipair_out_id_theta-1)))<=0.0d0).and.(((y2_theta-icenode0(2,ipair_out_id_theta))*(y2_theta-icenode0(2,ipair_out_id_theta-1)))<=0.0d0).and.(dabs(t1_theta-t2_theta)>zero)) then
                        write(*,*) 'em18'        ! two intersctions between section and circle
                        !vstops = stpi-1
                        !pause
                        !goto 111
                    else
                        new_ice_out(1,i) = x1_theta
                        new_ice_out(2,i) = y1_theta
                    end if
                else
                    if ((((x2_theta-icenode0(1,ipair_out_id_theta))*(x2_theta-icenode0(1,ipair_out_id_theta-1)))<=0.0d0).and.(((y2_theta-icenode0(2,ipair_out_id_theta))*(y2_theta-icenode0(2,ipair_out_id_theta-1)))<=0.0d0)) then
                        new_ice_out(1,i) = x2_theta
                        new_ice_out(2,i) = y2_theta
                    else
                        write(*,*) 'em19'
                        !vstops = stpi-1
                        !pause
                        !goto 111
                    end if
                end if
            end if
                            
            ! opening angle
            
            Mid_theta(1) = (intsct_in(1,i)+intsct_out(1,i))/2.0d0
            Mid_theta(2) = (intsct_in(2,i)+intsct_out(2,i))/2.0d0                
            
            ! circle center for opening angle, theta, pf
            if (cusp_id(i)>0) then
                if (dabs(intsct_in(1,i)-intsct_out(1,i))<=zero4) then
                    c_theta(1) = icenode0(1,cusp_id(i))
                    c_theta(2) = Mid_theta(2)
                else if (dabs(intsct_in(2,i)-intsct_out(2,i))<=zero4) then
                    c_theta(1) = Mid_theta(1)
                    c_theta(2) = icenode0(2,cusp_id(i))
                else
                    k_theta = (intsct_in(2,i)-intsct_out(2,i))/(intsct_in(1,i)-intsct_out(1,i))
                    kp_theta = -1.0d0/k_theta
                    c_theta(1) = (kp_theta*kp_theta*Mid_theta(1)+kp_theta*(icenode0(2,cusp_id(i))-Mid_theta(2))+icenode0(1,cusp_id(i)))/(kp_theta*kp_theta+1.0d0)
                    c_theta(2) = kp_theta*(c_theta(1)-Mid_theta(1))+Mid_theta(2)
                end if
                v_in_theta(1) = new_ice_in(1,i)-c_theta(1)      ! vector between new_ice_in and c_theta
                v_in_theta(2) = new_ice_in(2,i)-c_theta(2)
                v_out_theta(1) = new_ice_out(1,i)-c_theta(1)
                v_out_theta(2) = new_ice_out(2,i)-c_theta(2)
                
                theta_in = kai(v_in_theta(2),v_in_theta(1))
                theta_out = kai(v_out_theta(2),v_out_theta(1))
                if (theta_out > theta_in) then
                    theta_wedge(i) = theta_out - theta_in
                else
                    theta_wedge(i) = theta_out - theta_in + 2.0d0*pi
                end if
                
                lamda = 1.64601d0+2.46879d0*((0.5d0*(vnfip+vnlip))**0.39958d0)
                pf = lamda*(theta_wedge(i)/pi)*(theta_wedge(i)/pi)*sigmaf*hi**2.0d0
            else
                pf = 1.0d20
            end if
            
            !if (stpi==900000000) then
            !    open(787,file='pf-info.txt')
            !    write(787,*) 'pf:',pf
            !    close(787)
            !end if                    
                            
            if(fvt_main(i)>pf) then
                ! center of break
                M_cenbrk(1,i) = (new_ice_in(1,i)+new_ice_out(1,i))/2.0d0
                M_cenbrk(2,i) = (new_ice_in(2,i)+new_ice_out(2,i))/2.0d0
                
                ! identify the arc center                
                if (cusp_id(i)>0) then
                    if (dabs(new_ice_in(1,i)-new_ice_out(1,i))<=zero4) then
                        c_cenbrk(1) = icenode0(1,cusp_id(i))
                        c_cenbrk(2) = M_cenbrk(2,i)
                    else if (dabs(new_ice_in(2,i)-new_ice_out(2,i))<=zero4) then
                        c_cenbrk(1) = M_cenbrk(1,i)
                        c_cenbrk(2) = icenode0(2,cusp_id(i))
                    else
                        k_cenbrk = (new_ice_in(2,i)-new_ice_out(2,i))/(new_ice_in(1,i)-new_ice_out(1,i))
                        kp_cenbrk = -1.0d0/k_cenbrk
                        !c_cenbrk(1) = (icenode0(2,cusp_id(i))*(new_ice_in(2,i)-new_ice_out(2,i))-0.5d0*(new_ice_in(1,i)*new_ice_in(1,i)+new_ice_in(2,i)*new_ice_in(2,i)-new_ice_out(1,i)*new_ice_out(1,i)-new_ice_out(2,i)*new_ice_out(2,i))-icenode0(1,cusp_id(i))*(new_ice_in(2,i)-new_ice_out(2,i))*(new_ice_in(2,i)-new_ice_out(2,i))/(new_ice_in(1,i)-new_ice_out(1,i)))/(new_ice_out(1,i)-new_ice_in(1,i)-(new_ice_in(2,i)-new_ice_out(2,i))*(new_ice_in(2,i)-new_ice_out(2,i))/(new_ice_in(1,i)-new_ice_out(1,i)))
                        !c_cenbrk(2) = (new_ice_in(2,i)-new_ice_out(2,i))*(c_cenbrk(1)-icenode0(1,cusp_id(i)))/(new_ice_in(1,i)-new_ice_out(1,i))+icenode0(2,cusp_id(i))
                        c_cenbrk(1) = (kp_cenbrk*kp_cenbrk*M_cenbrk(1,i)+kp_cenbrk*(icenode0(2,cusp_id(i))-M_cenbrk(2,i))+icenode0(1,cusp_id(i)))/(kp_cenbrk*kp_cenbrk+1.0d0)
                        c_cenbrk(2) = kp_cenbrk*(c_cenbrk(1)-M_cenbrk(1,i))+M_cenbrk(2,i)
                        !c_cenbrk(1) = (icenode0(2,cusp_id(i))-icenode0(1,cusp_id(i))*k_cenbrk-(M_cenbrk(2,i)-M_cenbrk(1,i)*kp_cenbrk))/(kp_cenbrk-k_cenbrk)
                        !c_cenbrk(2) = kp_cenbrk*c_cenbrk(1)+(M_cenbrk(2,i)-M_cenbrk(1,i)*kp_cenbrk)                    ! circle center for bending crack
                    end if
                else
                    write(*,*) 'wm23'
                    !pause
                endif
                
                in_pt(1) = new_ice_in(1,i)
                in_pt(2) = new_ice_in(2,i)
                out_pt(1) = new_ice_out(1,i)
                out_pt(2) = new_ice_out(2,i)
                
                ! correction of breaking center
                dist_io = dsqrt((in_pt(1)-out_pt(1))*(in_pt(1)-out_pt(1))+(in_pt(2)-out_pt(2))*(in_pt(2)-out_pt(2)))
                
                if (dist_io>=10.0d0*thousand) then
                dist_flag = 1
                end if
				h_mln = 0.5d0*kc*dist_io*(0.25d0-1.0d0/(kc*kc))
                
                if (dabs(new_ice_in(1,i)-new_ice_out(1,i))<=zero4) then
                    c_cenbrk_1(1) = M_cenbrk(1,i) + h_mln
                    c_cenbrk_1(2) = M_cenbrk(2,i)
                    c_cenbrk_2(1) = M_cenbrk(1,i) - h_mln
                    c_cenbrk_2(2) = M_cenbrk(2,i)
                else if (dabs(new_ice_in(2,i)-new_ice_out(2,i))<=zero4) then
                    c_cenbrk_1(1) = M_cenbrk(1,i)
                    c_cenbrk_1(2) = M_cenbrk(2,i) + h_mln
                    c_cenbrk_2(1) = M_cenbrk(1,i)
                    c_cenbrk_2(2) = M_cenbrk(2,i) - h_mln
                else
                    k_cenbrk = (new_ice_in(2,i)-new_ice_out(2,i))/(new_ice_in(1,i)-new_ice_out(1,i))
                    kp_cenbrk = -1.0d0/k_cenbrk
                    c_cenbrk_1(1) = M_cenbrk(1,i) + h_mln/dsqrt(kp_cenbrk*kp_cenbrk+1.0d0)
                    c_cenbrk_1(2) = M_cenbrk(2,i) + kp_cenbrk*h_mln/dsqrt(kp_cenbrk*kp_cenbrk+1.0d0)
                    c_cenbrk_2(1) = M_cenbrk(1,i) - h_mln/dsqrt(kp_cenbrk*kp_cenbrk+1.0d0)
                    c_cenbrk_2(2) = M_cenbrk(2,i) - kp_cenbrk*h_mln/dsqrt(kp_cenbrk*kp_cenbrk+1.0d0)
                end if	
                
                if (((c_cenbrk_1(1)-new_ice_out(1,i))*(new_ice_in(2,i)-new_ice_out(2,i))-(new_ice_in(1,i)-new_ice_out(1,i))*(c_cenbrk_1(2)-new_ice_out(2,i)))>=0.0d0) then
                    c_cenbrk(1) = c_cenbrk_1(1)
                    c_cenbrk(2) = c_cenbrk_1(2)
                else
                    c_cenbrk(1) = c_cenbrk_2(1)
                    c_cenbrk(2) = c_cenbrk_2(2)
                end if                    
                
                dist_ctrl = 0
                if (dist_io>=10.0d0*thousand.and.dist_ctrl == 1) then
                    write(*,*) 'em24: dist_io out of range'
                    !pause
                    open (02002,file = 'dist_io out of range.txt')
                        write(02002,*) 'dist_io=',dist_io
                        !write(02002,*) 'stpi:',stpi
                        write(02002,*) 'c_rgns,i,cusp_id:',c_rgns,i,cusp_id(i)
                        write(02002,*) 'ipair_in_id(i),ipair_out_id(i),hpair_in_id(i),hpair_out_id(i):',ipair_in_id(i),ipair_out_id(i),hpair_in_id(i),hpair_out_id(i)
                        write(02002,*) 'intsct_in(1,i), intsct_in(2,i):',intsct_in(1,i), intsct_in(2,i)
                        write(02002,*) 'intsct_out(1,i), intsct_out(2,i):',intsct_out(1,i), intsct_out(2,i)
                    close(02002)
                    open(458780,file='ice-m24.txt')
                        do j = 1,icenode_no_0
                            write(458780,*) icenode0(1,j),icenode0(2,j)
                        end do
                    close(458780)
                    open(789930,file='ship-m24.txt')
                        do j = 1,wl_dim
                            write(789930,*) xyoutwl(1,j),xyoutwl(2,j)
                        end do
                            write(789930,*) xyoutwl(1,1),xyoutwl(2,1)
                    close(789930)
                    stop
                end if
                
                r_check = dsqrt((in_pt(1)-c_cenbrk(1))*(in_pt(1)-c_cenbrk(1))+(in_pt(2)-c_cenbrk(2))*(in_pt(2)-c_cenbrk(2)))                     
                
                call newedge(in_pt,out_pt,c_cenbrk,dl,n_new_pt,new_edge,iceupdt_flag)
                
                if (iceupdt_flag == 1) then
                    open(3232323,file='ice_e02.txt')
                        do j = 1,icenode_no_0
                            write(3232323,*) icenode0(1,j),icenode0(2,j)
                        end do
                    close(3232323)
                    open(78993011,file='ship_e02.txt')
                        do j = 1,wl_dim
                            write(78993011,*) xyoutwl(1,j),xyoutwl(2,j)
                        end do
                            write(78993011,*) xyoutwl(1,1),xyoutwl(2,1)
                    close(78993011)
                    write(*,*) 'i:',i
                    write(*,*) 'c_rgns:',c_rgns
                    write(*,*) 'writing results done!'
                    !pause
                    stop
                end if
                	                
                icenode_no = icenode_no_0-(ipair_out_id_theta-ipair_in_id_theta-1)+2+n_new_pt
                
                !update ice nodes
                forall(k = 1:2,j = 1:ipair_in_id_theta)
                    icenode(k,j) = icenode0(k,j)
                end forall
                
                forall(k = 1:2)
                    icenode(k,ipair_in_id_theta+1) = new_ice_in(k,i)
                end forall
                
                forall(k = 1:2,j = (ipair_in_id_theta+2):(ipair_in_id_theta+n_new_pt+1))
                    icenode(k,j) = new_edge(k,j-ipair_in_id_theta-1)
                end forall
                
                forall(k = 1:2)
                    icenode(k,ipair_in_id_theta+n_new_pt+2) = new_ice_out(k,i)
                end forall
                
                forall(k = 1:2,j = (ipair_in_id_theta+n_new_pt+3):icenode_no)      ! the use of icenode_no and the next line
                    icenode(k,j) = icenode0(k,j-icenode_no+icenode_no_0)
                end forall
                
                r_check_ctrl = 0
                if (r_check>10.0d0*thousand.and.dist_flag == 0.and.r_check_ctrl == 1) then
                    write(*,*) ' em25 '
                    !pause
                    open(4578,file='r_check.txt')
                    !write(4578,*) 'stpi,i:',stpi,i
                    write(4578,*) 'c_rgns, cusp_id(i):',c_rgns, cusp_id(i)
                    write(4578,*) 'ipair_in_id(i),ipair_out_id(i),hpair_in_id(i),hpair_out_id(i):',ipair_in_id(i),ipair_out_id(i),hpair_in_id(i),hpair_out_id(i)
                    write(4578,*) 'intsct_in(1,i), intsct_in(2,i):',intsct_in(1,i), intsct_in(2,i)
                    write(4578,*) 'intsct_out(1,i), intsct_out(2,i):',intsct_out(1,i), intsct_out(2,i)
                    close(4578)
                    open(45878,file='ice-m25-before.txt')
                    do j = 1,icenode_no_0
                        write(45878,*) icenode0(1,j),icenode0(2,j)
                    end do
                    close(45878)
                    open(34221,file='ice-m25-after.txt')
                    do j = 1,icenode_no
                        write(34221,*) icenode(1,j),icenode(2,j)
                    end do
                    close(34221)
                    open(78993,file='ship-m25.txt')
                    do j = 1,wl_dim
                        write(78993,*) xyoutwl(1,j),xyoutwl(2,j)
                    end do
                    close(78993)
                    !pause
                    stop
                end if	 
                  	                        	                
                icenode0 = icenode
                icenode_no_0 = icenode_no
                
                if ((c_rgns>=2).and.(i<c_rgns)) then
                    do j = i+1,c_rgns
                        ipair_in_id(j) = ipair_in_id(j) + (n_new_pt-(ipair_out_id_theta-ipair_in_id_theta-1)+2)
                        ipair_out_id(j) = ipair_out_id(j) + (n_new_pt-(ipair_out_id_theta-ipair_in_id_theta-1)+2)
                    end do
                end if
                    	                
                fshpsum = fshpsum + fshpsum_main*pf/fvt_main(i)
            else
                fshpsum = fshpsum + fshpsum_main
                !write(*,*) 'mark6'
            end if
                            
            !write(*,*) 'mark6'    
            !pause
        
        end if   ! level 2

    end do         
    ! do i = 1,c_rgns

    !write(*,*) 'mark7'
    !pause
    
    Fcrbrk = fshpsum

end if                     ! level 1
	F(1) = -Fcrbrk(2)
    F(2) = -Fcrbrk(1)
    F(3) = -Fcrbrk(3)
	M(1) = -Fcrbrk(5)
    M(2) = -Fcrbrk(4)
    M(3) = -Fcrbrk(6) 
              
!-----------------------------------------------------------------------------------------------------------   

dstern1 = dsqrt((icenode0(1,1)-xyoutwl(1,1))*(icenode0(1,1)-xyoutwl(1,1))+(icenode0(2,1)-xyoutwl(2,1))*(icenode0(2,1)-xyoutwl(2,1)))
dstern2 = dsqrt((icenode0(1,icenode_no_0)-xyoutwl(1,1))*(icenode0(1,icenode_no_0)-xyoutwl(1,1))+(icenode0(2,icenode_no_0)-xyoutwl(2,1))*(icenode0(2,icenode_no_0)-xyoutwl(2,1)))

if(dstern1>8.0d3.and.dstern2>8.0d3) then
    kh = 0
    kt = 0

    do i = 1,icenode_no_0
        dst_strn = dsqrt((icenode0(1,i)-xyoutwl(1,1))*(icenode0(1,i)-xyoutwl(1,1))+(icenode0(2,i)-xyoutwl(2,1))*(icenode0(2,i)-xyoutwl(2,1)))
        dst_cog = dsqrt((icenode0(1,i)-r(1)*thousand)*(icenode0(1,i)-r(1)*thousand)+(icenode0(2,i)-r(2)*thousand)*(icenode0(2,i)-r(2)*thousand))
        !if(dst_strn>3.0d3.and.dst_cog>55.0d3) then
        if (dst_strn>6.0d3) then
            kh = kh + 1
        else
            exit
        end if
    end do
    
    if(kh>0) then
        forall (i = 1:2, j = 1:kh)
            icenodeh(i,icenode_no_h+j) = icenode0(i,j)
        end forall
        icenode_no_h = icenode_no_h + kh
    end if

    do i = icenode_no_0,1,-1
        dst_strn = dsqrt((icenode0(1,i)-xyoutwl(1,1))*(icenode0(1,i)-xyoutwl(1,1))+(icenode0(2,i)-xyoutwl(2,1))*(icenode0(2,i)-xyoutwl(2,1)))
        dst_cog = dsqrt((icenode0(1,i)-r(1)*thousand)*(icenode0(1,i)-r(1)*thousand)+(icenode0(2,i)-r(2)*thousand)*(icenode0(2,i)-r(2)*thousand))
        !if(dst_strn>25.0d3.and.dst_cog>55.0d3) then
        if (dst_strn>6.0d3) then
            kt = kt + 1
        else
            exit
        end if
    end do

    if(kt>0) then
        forall (i = 1:2,j = 1:kt)
            icenodet(i,icenode_no_t+j) = icenode0(i,icenode_no-j+1)
        end forall
        icenode_no_t = icenode_no_t + kt
    end if

    if(kh>0.or.kt>0) then
        icenode_no_0 = icenode_no_0 - kt - kh
        forall (i = 1:2, j = 1:icenode_no_0)
            icenode0(i,j) = icenode0(i,j+kh)
        end forall
    end if

end if

open(fileid,file = 'ice_updated.txt',status = 'replace')
do i = 1, icenode_no_0
	write(fileid,*) icenode0(1,i), icenode0(2,i)
end do
close(fileid)
open(fileid,file = 'ice_nd_updated.txt',status = 'replace')
write(fileid,*) icenode_no_0
close(fileid)

open(fileid,file = 'ice_initial.txt',status = 'replace')
do i = 1, icenode_no_00
	write(fileid,*) icenode00(1,i), icenode00(2,i)
end do
close(fileid)
open(fileid,file = 'ice_nd_initial.txt', status = 'replace')
write(fileid,*) icenode_no_00
close(fileid)

!--------------------------------------------------------------- recordings
!do k=1,6
!    Rice(k,stpi) = Fcrbrk(k) + Fsub(k)
!    Fcrbrk_his(k,stpi) = Fcrbrk(k)
!end do
    
!if(mod(stpi,print_itvl)==0) then  
!    write(*,*)"t(s):",stpi/print_itvl,icenode_no_0
!    write(*,*)"i:",hi
!    write(*,*)"v:",v(1),r(6)/pi*180.0d0
!end if

!instantaneous output------------------------------------------------   

!if (mod(stpi,100000000000)==0) then
!    write(fileidd,"(i7)") ct
!    fileidd = adjustl(fileidd)
!    open(fileid,file=trim(fileidd)//'g.txt')

!    do k=1,wl_dim
!        write(fileid,'(2f20.5)')xyoutwl(1,k)/1000.0d0,xyoutwl(2,k)/1000.0d0
!    end do
!    write(fileid,'(2f20.5)')xyoutwl(1,1)/1000.0d0,xyoutwl(2,1)/1000.0d0
!    write(fileid,*)
!    if (icenode_no_h>0) then
!        do k=1,icenode_no_h
!            write(fileid,'(2f20.5)')icenodeh(1,k)/1000.0d0,icenodeh(2,k)/1000.0d0
!        end do
!    end if
!    do k = 1,icenode_no_0
!        write(fileid,'(2f20.5)')icenode0(1,k)/1000.0d0,icenode0(2,k)/1000.0d0
!    end do
!    if (icenode_no_t>0) then
!        do k = icenode_no_t,1,-1
!            write(fileid,'(2f20.5)')icenodet(1,k)/1000.0d0,icenodet(2,k)/1000.0d0
!        end do
!    end if
!    close(fileid)
!    ct = ct + 1

!  if((T.LT.100).AND.(ABS(((F(2)-1000.0)/1000.0).GT.0.05))) then
!  F(2)=1000.0
!  endif

endif





F1(1)=F(1)*0.001
F1(2)=F(2)*0.001
F1(3)=F(3)*0.001
M1(1)=M(1)*0.001
M1(2)=M(2)*0.001
M1(3)=M(3)*0.001
! close (100)
!write (*,"(f10.5)") F
open(3000, FILE="iceload_coupled_spar.txt") 
write (3000,'(f8.3,f20.5,f20.5,f20.5,f20.5,f20.5,f20.5)') time, F1(1), F1(2), F1(3), M1(1), M1(2), M1(3)

!
END SUBROUTINE ForceDLL


